// Variables
const llmHost = 'https://llm.vfsh.me';
const generateEndpoint = '/api/generate';

// Grab elements from the page
const userInput = document.getElementById('userInput');
const sendButton = document.getElementById('sendButton');
const outputArea = document.getElementById('outputArea');

// Functions
async function sendGen() {
    const response = await fetch(`${llmHost}${generateEndpoint}`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({ model: 'llama3.2', prompt: userInput.value })
    });

    return response;
}

async function readResponse(response) {
    const reader = response.body.getReader();
    const decoder = new TextDecoder('utf-8');

    try {
        // Recursive function to read the stream
        function read() {
            reader.read().then(({ value, done }) => {
                if (done) {
                    console.log("Stream complete");
                    return;
                }

                // Decode the stream chunk and split into lines
                const chunk = decoder.decode(value, { stream: true });
                const lines = chunk.split(/\r?\n/); // Split by newlines

                // Process each line
                for (const line of lines) {
                    if (line.trim()) { // Skip empty lines
                        try {
                            const json = JSON.parse(line); // Parse each line as JSON
                            if (json.response) {
                                outputArea.value += json.response; // Append response text to the outputArea
                            }
                        } catch (err) {
                            console.error("Failed to parse JSON:", err, line);
                        }
                    }
                }

                // Continue reading
                read();
            });
        }

        // Start reading
        read();
    } catch (error) {
        outputArea.innerHTML = `<span class="text-red-500">Error: ${error.message}</span>`;
    }
}

// Event listeners
sendButton.addEventListener('click', async () => {
    outputArea.innerHTML = '';
    const response = await sendGen();
    readResponse(response);
});